@@ -17,6 +17,7 @@ import logging
 from multiprocessing import managers
 from multiprocessing import util as mp_util
 import threading
+import time
 import weakref
 
 import oslo_rootwrap
@@ -41,17 +42,23 @@ except AttributeError:
 
 ClientManager = daemon.get_manager_class()
 LOG = logging.getLogger(__name__)
+SHUTDOWN_RETRIES = 3
 
 
 class Client(object):
     def __init__(self, rootwrap_daemon_cmd):
         self._start_command = rootwrap_daemon_cmd
         self._initialized = False
+        self._need_restart = False
         self._mutex = threading.Lock()
         self._manager = None
         self._proxy = None
         self._process = None
         self._finalize = None
+        # This is for eventlet compatibility. multiprocessing stores
+        # daemon connection in ForkAwareLocal, so this won't be
+        # needed with the threading module.
+        self._exec_sem = threading.Lock()
 
     def _initialize(self):
         if self._process is not None and self._process.poll() is not None:
@@ -62,7 +69,8 @@ class Client(object):
         process_obj = subprocess.Popen(self._start_command,
                                        stdin=subprocess.PIPE,
                                        stdout=subprocess.PIPE,
-                                       stderr=subprocess.PIPE)
+                                       stderr=subprocess.PIPE,
+                                       close_fds=True)
         LOG.debug("Popen for %s command has been instantiated",
                   self._start_command)
 
@@ -79,6 +87,15 @@ class Client(object):
                             (stderr,))
         LOG.info("Spawned new rootwrap daemon process with pid=%d",
                  process_obj.pid)
+
+        def wait_process():
+            return_code = process_obj.wait()
+            LOG.info("Rootwrap daemon process exit with status: %d",
+                     return_code)
+
+        reap_process = threading.Thread(target=wait_process)
+        reap_process.daemon = True
+        reap_process.start()
         self._manager = ClientManager(socket_path, authkey)
         self._manager.connect()
         self._proxy = self._manager.rootwrap()
@@ -93,10 +110,14 @@ class Client(object):
         if process.poll() is None:
             LOG.info('Stopping rootwrap daemon process with pid=%s',
                      process.pid)
-            try:
-                manager.rootwrap().shutdown()
-            except (EOFError, IOError):
-                pass  # assume it is dead already
+            for _ in range(SHUTDOWN_RETRIES):
+                try:
+                    manager.rootwrap().shutdown()
+                    break
+                except (EOFError, IOError):
+                    break  # assume it is dead already
+                except RuntimeError:
+                    time.sleep(0.2)
             # We might want to wait for process to exit or kill it, but we
             # can't provide sane timeout on 2.x and we most likely don't have
             # permisions to do so
@@ -110,7 +131,8 @@ class Client(object):
 
     def _restart(self, proxy):
         with self._mutex:
-            assert self._initialized
+            if not self._initialized:
+                raise AssertionError("Client should be initialized.")
             # Verify if someone has already restarted this.
             if self._proxy is proxy:
                 self._finalize()
@@ -118,20 +140,40 @@ class Client(object):
                 self._proxy = None
                 self._initialized = False
                 self._initialize()
+                self._need_restart = False
             return self._proxy
 
-    def execute(self, cmd, stdin=None):
-        self._ensure_initialized()
-        proxy = self._proxy
-        retry = False
+    def _run_one_command(self, proxy, cmd, stdin):
+        """Wrap proxy.run_one_command, setting _need_restart on an exception.
+
+        Usually it should be enough to drain stale data on socket
+        rather than to restart, but we cannot do draining easily.
+        """
         try:
+            _need_restart = True
             res = proxy.run_one_command(cmd, stdin)
-        except (EOFError, IOError):
-            retry = True
-        # res can be None if we received final None sent by dying server thread
-        # instead of response to our request. Process is most likely to be dead
-        # at this point.
-        if retry or res is None:
-            proxy = self._restart(proxy)
-            res = proxy.run_one_command(cmd, stdin)
+            _need_restart = False
+            return res
+        finally:
+            if _need_restart:
+                self._need_restart = True
+
+    def execute(self, cmd, stdin=None):
+        with self._exec_sem:
+            self._ensure_initialized()
+            proxy = self._proxy
+            retry = False
+            if self._need_restart:
+                proxy = self._restart(proxy)
+            try:
+                res = self._run_one_command(proxy, cmd, stdin)
+            except (EOFError, IOError):
+                retry = True
+            # res can be None if we received final None sent by dying
+            # server thread instead of response to our
+            # request. Process is most likely to be dead at this
+            # point.
+            if retry or res is None:
+                proxy = self._restart(proxy)
+                res = self._run_one_command(proxy, cmd, stdin)
         return res
